@@ -3,6 +3,7 @@ export RUSTFLAGS=-Zinstrument-coverage
 export CARGO_INCREMENTAL=0
 
 rustup default nightly
+rustup target add x86_64-pc-windows-gnu
 cargo build
 cargo test
 unset RUSTFLAGS
@@ -22,4 +23,6 @@ target/debug/cargo-cinstall --help
 target/debug/cargo-cinstall cinstall --manifest-path=example-project/Cargo.toml --destdir=/tmp/staging
 target/debug/cargo-cinstall cinstall clean --manifest-path=example-project/Cargo.toml
 
+target/debug/cargo-cinstall cinstall --manifest-path=example-project/Cargo.toml --destdir=/tmp/staging-win --target=x86_64-pc-windows-gnu
+
 grcov . --binary-path target/debug/deps/ -s . -t lcov --branch --ignore-not-existing --ignore '../**' --ignore '/*' -o coverage.lcov
