@@ -1,6 +1,4 @@
 use std::collections::HashMap;
-use std::env;
-use std::fs::File;
 use std::io::{BufRead, BufReader, Read, Write};
 use std::path::{Path, PathBuf};
 use std::sync::atomic::{AtomicBool, Ordering};
@@ -13,10 +11,11 @@ use cargo::core::{compiler::Executor, profiles::Profiles};
 use cargo::core::{TargetKind, Workspace};
 use cargo::ops::{self, CompileFilter, CompileOptions, FilterRule, LibRule};
 use cargo::util::command_prelude::{ArgMatches, ArgMatchesExt, CompileMode, ProfileChecking};
+use cargo::util::interning::InternedString;
 use cargo::{CliError, CliResult, Config};
 
 use anyhow::Error;
-use cargo_util::paths::copy;
+use cargo_util::paths::{copy, create, create_dir_all, open, read, read_bytes, write};
 use semver::Version;
 
 use crate::build_targets::BuildTargets;
@@ -69,10 +68,10 @@ fn copy_prebuilt_include_file(
 ) -> anyhow::Result<()> {
     ws.config()
         .shell()
-        .status("Copying", "pre-built header files")?;
+        .status("Populating", "uninstalled header directory")?;
     for (from, to) in build_targets.extra.include.iter() {
-        let to = root_output.join(to);
-        std::fs::create_dir_all(to.parent().unwrap())?;
+        let to = root_output.join("include").join(to);
+        create_dir_all(to.parent().unwrap())?;
         copy(from, to)?;
     }
 
@@ -81,14 +80,9 @@ fn copy_prebuilt_include_file(
 
 fn build_pc_file(name: &str, root_output: &Path, pc: &PkgConfig) -> anyhow::Result<()> {
     let pc_path = root_output.join(&format!("{}.pc", name));
-
-    let mut out = std::fs::File::create(pc_path)?;
-
     let buf = pc.render();
 
-    out.write_all(buf.as_ref())?;
-
-    Ok(())
+    write(pc_path, buf)
 }
 
 fn build_pc_files(
@@ -99,7 +93,7 @@ fn build_pc_files(
 ) -> anyhow::Result<()> {
     ws.config().shell().status("Building", "pkg-config files")?;
     build_pc_file(filename, root_output, pc)?;
-    let pc_uninstalled = pc.uninstalled(&root_output);
+    let pc_uninstalled = pc.uninstalled(root_output);
     build_pc_file(
         &format!("{}-uninstalled", filename),
         root_output,
@@ -108,13 +102,12 @@ fn build_pc_files(
 }
 
 fn patch_target(
-    ws: &mut Workspace,
+    pkg: &mut Package,
     libkinds: &[&str],
     capi_config: &CApiConfig,
 ) -> anyhow::Result<()> {
     use cargo::core::compiler::CrateType;
 
-    let pkg = ws.current_mut()?;
     let manifest = pkg.manifest_mut();
     let targets = manifest.targets_mut();
 
@@ -139,19 +132,6 @@ fn patch_target(
     Ok(())
 }
 
-fn patch_capi_feature(compile_opts: &mut CompileOptions, ws: &Workspace) -> anyhow::Result<()> {
-    let pkg = ws.current()?;
-    let manifest = pkg.manifest();
-
-    if manifest.summary().features().get("capi").is_some() {
-        std::rc::Rc::get_mut(&mut compile_opts.cli_features.features)
-            .unwrap()
-            .insert(FeatureValue::new("capi".into()));
-    }
-
-    Ok(())
-}
-
 /// Build def file for windows-msvc
 fn build_def_file(
     ws: &Workspace,
@@ -182,9 +162,9 @@ fn build_def_file(
 
         let out = dumpbin.output()?;
         if out.status.success() {
-            let txt_file = File::open(txt_path)?;
+            let txt_file = open(txt_path)?;
             let buf_reader = BufReader::new(txt_file);
-            let mut def_file = File::create(targetdir.join(format!("{}.def", name)))?;
+            let mut def_file = create(targetdir.join(format!("{}.def", name)))?;
             writeln!(def_file, "{}", "EXPORTS".to_string())?;
 
             // The Rust loop below is analogue to the following loop.
@@ -298,12 +278,13 @@ fn build_implib_file(
     }
 }
 
-struct FingerPrint<'a> {
-    name: &'a str,
-    root_output: &'a Path,
+#[derive(Debug)]
+struct FingerPrint {
+    id: PackageId,
+    root_output: PathBuf,
     build_targets: BuildTargets,
-    install_paths: &'a InstallPaths,
-    static_libs: &'a str,
+    install_paths: InstallPaths,
+    static_libs: String,
 }
 
 #[derive(serde::Serialize, serde::Deserialize)]
@@ -312,19 +293,19 @@ struct Cache {
     static_libs: String,
 }
 
-impl<'a> FingerPrint<'a> {
+impl FingerPrint {
     fn new(
-        name: &'a str,
-        root_output: &'a Path,
-        build_targets: BuildTargets,
-        install_paths: &'a InstallPaths,
+        id: &PackageId,
+        root_output: &Path,
+        build_targets: &BuildTargets,
+        install_paths: &InstallPaths,
     ) -> Self {
         Self {
-            name,
-            root_output,
-            build_targets,
-            install_paths,
-            static_libs: "",
+            id: id.to_owned(),
+            root_output: root_output.to_owned(),
+            build_targets: build_targets.clone(),
+            install_paths: install_paths.clone(),
+            static_libs: String::new(),
         }
     }
 
@@ -337,16 +318,13 @@ impl<'a> FingerPrint<'a> {
 
         let mut paths: Vec<&PathBuf> = Vec::new();
         if let Some(include) = &self.build_targets.include {
-            paths.push(&include);
+            paths.push(include);
         }
         paths.extend(&self.build_targets.static_lib);
         paths.extend(&self.build_targets.shared_lib);
 
         for path in paths.iter() {
-            if let Ok(mut f) = std::fs::File::open(path) {
-                let mut buf = Vec::new();
-                f.read_to_end(&mut buf)?;
-
+            if let Ok(buf) = read_bytes(path) {
                 hasher.write(&buf);
             } else {
                 return Ok(None);
@@ -363,11 +341,11 @@ impl<'a> FingerPrint<'a> {
         // Use the crate name in the cache file as the same target dir
         // may be used to build various libs
         self.root_output
-            .join(format!("cargo-c-{}.cache", self.name))
+            .join(format!("cargo-c-{}.cache", self.id.name()))
     }
 
     fn load_previous(&self) -> anyhow::Result<Cache> {
-        let mut f = std::fs::File::open(&self.path())?;
+        let mut f = open(&self.path())?;
         let mut cache_str = String::new();
         f.read_to_string(&mut cache_str)?;
         let cache = toml::de::from_str(&cache_str)?;
@@ -383,21 +361,20 @@ impl<'a> FingerPrint<'a> {
     }
 
     fn store(&self) -> anyhow::Result<()> {
-        let mut f = std::fs::File::create(&self.path())?;
-
         if let Some(hash) = self.hash()? {
             let cache = Cache {
                 hash,
                 static_libs: self.static_libs.to_owned(),
             };
             let buf = toml::ser::to_vec(&cache)?;
-            f.write_all(&buf)?;
+            write(&self.path(), buf)?;
         }
 
         Ok(())
     }
 }
 
+#[derive(Debug)]
 pub struct CApiConfig {
     pub header: HeaderCApiConfig,
     pub pkg_config: PkgConfigCApiConfig,
@@ -405,6 +382,7 @@ pub struct CApiConfig {
     pub install: InstallCApiConfig,
 }
 
+#[derive(Debug)]
 pub struct HeaderCApiConfig {
     pub name: String,
     pub subdirectory: String,
@@ -412,6 +390,7 @@ pub struct HeaderCApiConfig {
     pub enabled: bool,
 }
 
+#[derive(Debug)]
 pub struct PkgConfigCApiConfig {
     pub name: String,
     pub filename: String,
@@ -422,6 +401,7 @@ pub struct PkgConfigCApiConfig {
     pub strip_include_path_components: usize,
 }
 
+#[derive(Debug)]
 pub struct LibraryCApiConfig {
     pub name: String,
     pub version: Version,
@@ -430,16 +410,18 @@ pub struct LibraryCApiConfig {
     pub rustflags: Vec<String>,
 }
 
+#[derive(Debug)]
 pub struct InstallCApiConfig {
     pub include: Vec<InstallTarget>,
 }
 
+#[derive(Debug)]
 pub enum InstallTarget {
     Asset(InstallTargetPaths),
     Generated(InstallTargetPaths),
 }
 
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct InstallTargetPaths {
     /// pattern to feed to glob::glob()
     ///
@@ -501,15 +483,16 @@ impl InstallTargetPaths {
     }
 }
 
-fn load_manifest_capi_config(
-    name: &str,
-    root_path: &Path,
-    ws: &Workspace,
-) -> anyhow::Result<CApiConfig> {
-    let mut manifest = std::fs::File::open(root_path.join("Cargo.toml"))?;
-    let mut manifest_str = String::new();
-    manifest.read_to_string(&mut manifest_str)?;
-
+fn load_manifest_capi_config(pkg: &Package) -> anyhow::Result<CApiConfig> {
+    let name = &pkg
+        .manifest()
+        .targets()
+        .iter()
+        .find(|t| t.is_lib())
+        .unwrap()
+        .crate_name();
+    let root_path = pkg.root().to_path_buf();
+    let manifest_str = read(&root_path.join("Cargo.toml"))?;
     let toml = manifest_str.parse::<toml::Value>()?;
 
     let capi = toml
@@ -551,7 +534,7 @@ fn load_manifest_capi_config(
         })
         .unwrap_or_else(|| Ok(String::from(name)))?;
 
-    let header = if let Some(ref capi) = capi {
+    let header = if let Some(capi) = capi {
         HeaderCApiConfig {
             name: header
                 .as_ref()
@@ -581,7 +564,6 @@ fn load_manifest_capi_config(
     };
 
     let pc = capi.and_then(|v| v.get("pkg_config"));
-    let pkg = ws.current().unwrap();
     let mut pc_name = String::from(name);
     let mut pc_filename = String::from(name);
     let mut description = String::from(
@@ -596,7 +578,7 @@ fn load_manifest_capi_config(
     let mut requires_private = None;
     let mut strip_include_path_components = 0;
 
-    if let Some(ref pc) = pc {
+    if let Some(pc) = pc {
         if let Some(override_name) = pc.get("name").and_then(|v| v.as_str()) {
             pc_name = String::from(override_name);
         }
@@ -638,7 +620,7 @@ fn load_manifest_capi_config(
     let mut versioning = true;
     let mut rustflags = Vec::new();
 
-    if let Some(ref library) = library {
+    if let Some(library) = library {
         if let Some(override_name) = library.get("name").and_then(|v| v.as_str()) {
             lib_name = String::from(override_name);
         }
@@ -675,8 +657,14 @@ fn load_manifest_capi_config(
         to: header.subdirectory.clone(),
     };
 
+    let header_name = if header.name.ends_with(".h") {
+        format!("assets/{}", header.name)
+    } else {
+        format!("assets/{}.h", header.name)
+    };
+
     let default_legacy_asset_include = InstallTargetPaths {
-        from: format!("assets/{}.h", header.name),
+        from: header_name,
         to: header.subdirectory.clone(),
     };
 
@@ -692,7 +680,7 @@ fn load_manifest_capi_config(
     ];
 
     let install = capi.and_then(|v| v.get("install"));
-    if let Some(ref install) = install {
+    if let Some(install) = install {
         if let Some(includes) = install.get("include") {
             if let Some(assets) = includes.get("asset").and_then(|v| v.as_array()) {
                 for asset in assets {
@@ -726,17 +714,18 @@ fn compile_options(
     ws: &Workspace,
     config: &Config,
     args: &ArgMatches<'_>,
-    default_profile: &str,
+    profile: InternedString,
     compile_mode: CompileMode,
 ) -> anyhow::Result<CompileOptions> {
     use cargo::core::compiler::CompileKind;
     let mut compile_opts =
         args.compile_options(config, compile_mode, Some(ws), ProfileChecking::Checked)?;
 
-    compile_opts.build_config.requested_profile =
-        args.get_profile_name(config, default_profile, ProfileChecking::Checked)?;
+    compile_opts.build_config.requested_profile = profile;
 
-    patch_capi_feature(&mut compile_opts, ws)?;
+    std::rc::Rc::get_mut(&mut compile_opts.cli_features.features)
+        .unwrap()
+        .insert(FeatureValue::new("capi".into()));
 
     compile_opts.filter = CompileFilter::new(
         LibRule::True,
@@ -883,18 +872,56 @@ fn compile_with_exec<'a>(
     Ok(out_dir)
 }
 
+#[derive(Debug)]
+pub struct CPackage {
+    pub version: Version,
+    pub root_path: PathBuf,
+    pub capi_config: CApiConfig,
+    pub build_targets: BuildTargets,
+    pub install_paths: InstallPaths,
+    finger_print: FingerPrint,
+}
+
+impl CPackage {
+    fn from_package(
+        pkg: &mut Package,
+        args: &ArgMatches<'_>,
+        libkinds: &[&str],
+        rustc_target: &target::Target,
+        root_output: &Path,
+    ) -> anyhow::Result<CPackage> {
+        let id = pkg.package_id();
+        let version = pkg.version().clone();
+        let root_path = pkg.root().to_path_buf();
+        let capi_config = load_manifest_capi_config(pkg)?;
+
+        patch_target(pkg, libkinds, &capi_config)?;
+
+        let name = &capi_config.library.name;
+
+        let install_paths = InstallPaths::new(name, args, &capi_config);
+        let build_targets =
+            BuildTargets::new(name, rustc_target, root_output, libkinds, &capi_config)?;
+
+        let finger_print = FingerPrint::new(&id, root_output, &build_targets, &install_paths);
+
+        Ok(CPackage {
+            version,
+            root_path,
+            capi_config,
+            build_targets,
+            install_paths,
+            finger_print,
+        })
+    }
+}
+
 pub fn cbuild(
     ws: &mut Workspace,
     config: &Config,
     args: &ArgMatches<'_>,
     default_profile: &str,
-) -> anyhow::Result<(
-    BuildTargets,
-    InstallPaths,
-    CApiConfig,
-    String,
-    CompileOptions,
-)> {
+) -> anyhow::Result<(Vec<CPackage>, CompileOptions)> {
     let rustc = config.load_global_rustc(Some(ws))?;
     let targets = args.targets();
     let target = match targets.len() {
@@ -915,27 +942,11 @@ pub fn cbuild(
     );
     let only_staticlib = !libkinds.contains(&"cdylib");
 
-    let name = &ws
-        .current()?
-        .manifest()
-        .targets()
-        .iter()
-        .find(|t| t.is_lib())
-        .unwrap()
-        .crate_name();
-    let version = ws.current()?.version().clone();
-    let root_path = ws.current()?.root().to_path_buf();
-    let capi_config = load_manifest_capi_config(name, &root_path, &ws)?;
+    let profile = args.get_profile_name(config, default_profile, ProfileChecking::Checked)?;
 
-    patch_target(ws, &libkinds, &capi_config)?;
+    let profiles = Profiles::new(ws, profile)?;
 
-    let name = &capi_config.library.name;
-
-    let install_paths = InstallPaths::new(name, args, &capi_config);
-
-    let mut compile_opts = compile_options(ws, config, args, default_profile, CompileMode::Build)?;
-
-    let profiles = Profiles::new(&ws, compile_opts.build_config.requested_profile)?;
+    let mut compile_opts = compile_options(ws, config, args, profile, CompileMode::Build)?;
 
     // TODO: there must be a simpler way to get the right path.
     let root_output = ws
@@ -945,13 +956,17 @@ pub fn cbuild(
         .join(PathBuf::from(target))
         .join(&profiles.get_dir_name());
 
+    let pkg = ws.current_mut()?;
+
+    let mut cpkg = CPackage::from_package(pkg, args, &libkinds, &rustc_target, &root_output)?;
+
     let mut leaf_args: Vec<String> = rustc_target
-        .shared_object_link_args(&capi_config, &install_paths.libdir, &root_output)
+        .shared_object_link_args(&cpkg.capi_config, &cpkg.install_paths.libdir, &root_output)
         .into_iter()
         .flat_map(|l| vec!["-C".to_string(), format!("link-arg={}", l)])
         .collect();
 
-    leaf_args.extend(capi_config.library.rustflags.clone());
+    leaf_args.extend(cpkg.capi_config.library.rustflags.clone());
 
     leaf_args.push("--cfg".into());
     leaf_args.push("cargo_c".into());
@@ -964,13 +979,7 @@ pub fn cbuild(
         leaf_args.push("target-feature=+crt-static".into());
     }
 
-    let mut build_targets =
-        BuildTargets::new(&name, &rustc_target, &root_output, &libkinds, &capi_config)?;
-
-    let mut finger_print =
-        FingerPrint::new(&name, &root_output, build_targets.clone(), &install_paths);
-
-    let pristine = finger_print.load_previous().is_err();
+    let pristine = cpkg.finger_print.load_previous().is_err();
 
     if pristine {
         // If the cache is somehow missing force a full rebuild;
@@ -983,19 +992,19 @@ pub fn cbuild(
         &compile_opts,
         &(exec.clone() as Arc<dyn Executor>),
         &leaf_args,
-        &capi_config.library.rustflags,
+        &cpkg.capi_config.library.rustflags,
     )?;
 
-    build_targets
+    cpkg.build_targets
         .extra
-        .setup(&capi_config, &root_path, out_dir.as_deref())?;
+        .setup(&cpkg.capi_config, &cpkg.root_path, out_dir.as_deref())?;
 
-    if capi_config.header.generation {
-        let mut header_name = PathBuf::from(&capi_config.header.name);
+    if cpkg.capi_config.header.generation {
+        let mut header_name = PathBuf::from(&cpkg.capi_config.header.name);
         header_name.set_extension("h");
         let from = root_output.join(&header_name);
-        let to = Path::new(&capi_config.header.subdirectory).join(&header_name);
-        build_targets.extra.include.push((from, to));
+        let to = Path::new(&cpkg.capi_config.header.subdirectory).join(&header_name);
+        cpkg.build_targets.extra.include.push((from, to));
     }
 
     if pristine {
@@ -1004,24 +1013,27 @@ pub fn cbuild(
     }
 
     let new_build = exec.ran.load(Ordering::Relaxed);
-    let mut static_libs = exec.link_line.lock().unwrap().clone();
+    let static_libs = exec.link_line.lock().unwrap().clone();
 
     // it is a new build, build the additional files and update update the cache
     // if the hash value does not match.
-    if new_build && !finger_print.is_valid() {
-        finger_print.static_libs = &static_libs;
+    if new_build && !cpkg.finger_print.is_valid() {
+        let name = &cpkg.capi_config.library.name;
+        let capi_config = &cpkg.capi_config;
+        let build_targets = &cpkg.build_targets;
+        cpkg.finger_print.static_libs = static_libs.to_owned();
 
-        let mut pc = PkgConfig::from_workspace(name, &install_paths, args, &capi_config);
+        let mut pc = PkgConfig::from_workspace(name, &cpkg.install_paths, args, capi_config);
         if only_staticlib {
             pc.add_lib(&static_libs);
         }
         pc.add_lib_private(&static_libs);
 
-        build_pc_files(&ws, &capi_config.pkg_config.filename, &root_output, &pc)?;
+        build_pc_files(ws, &capi_config.pkg_config.filename, &root_output, &pc)?;
 
         if !only_staticlib {
             let lib_name = name;
-            build_def_file(&ws, &lib_name, &rustc_target, &root_output)?;
+            build_def_file(ws, lib_name, &rustc_target, &root_output)?;
 
             let mut dlltool = std::env::var_os("DLLTOOL")
                 .map(PathBuf::from)
@@ -1032,16 +1044,22 @@ pub fn cbuild(
                 dlltool = args.value_of("dlltool").map(PathBuf::from).unwrap();
             }
 
-            build_implib_file(&ws, &lib_name, &rustc_target, &root_output, &dlltool)?;
+            build_implib_file(ws, lib_name, &rustc_target, &root_output, &dlltool)?;
         }
 
         if capi_config.header.enabled {
             let header_name = &capi_config.header.name;
             if capi_config.header.generation {
-                build_include_file(&ws, header_name, &version, &root_output, &root_path)?;
+                build_include_file(
+                    ws,
+                    header_name,
+                    &cpkg.version,
+                    &root_output,
+                    &cpkg.root_path,
+                )?;
             }
 
-            copy_prebuilt_include_file(&ws, &build_targets, &root_output)?;
+            copy_prebuilt_include_file(ws, build_targets, &root_output)?;
         }
 
         if name.contains('-') {
@@ -1050,7 +1068,7 @@ pub fn cbuild(
                 &rustc_target,
                 &root_output,
                 &libkinds,
-                &capi_config,
+                capi_config,
             )?;
 
             if let (Some(from_static_lib), Some(to_static_lib)) = (
@@ -1067,32 +1085,24 @@ pub fn cbuild(
             }
         }
 
-        finger_print.store()?;
+        cpkg.finger_print.store()?;
     } else {
         // It is not a new build, recover the static_libs value from the cache
-        static_libs = finger_print.load_previous()?.static_libs;
+        cpkg.finger_print.static_libs = cpkg.finger_print.load_previous()?.static_libs;
     }
 
-    Ok((
-        build_targets,
-        install_paths,
-        capi_config,
-        static_libs,
-        compile_opts,
-    ))
+    Ok((vec![cpkg], compile_opts))
 }
 
 pub fn ctest(
     ws: &Workspace,
-    _capi_config: &CApiConfig,
     config: &Config,
     args: &ArgMatches<'_>,
-    build_targets: BuildTargets,
-    static_libs: String,
+    packages: &[CPackage],
     mut compile_opts: CompileOptions,
 ) -> CliResult {
     compile_opts.build_config.requested_profile =
-        args.get_profile_name(&config, "test", ProfileChecking::Checked)?;
+        args.get_profile_name(config, "test", ProfileChecking::Checked)?;
     compile_opts.build_config.mode = CompileMode::Test;
 
     compile_opts.filter = ops::CompileFilter::new(
@@ -1116,26 +1126,32 @@ pub fn ctest(
     let test_args = test_args.collect::<Vec<_>>();
 
     use std::ffi::OsString;
-    let static_lib_path = build_targets.static_lib.unwrap();
-    let builddir = static_lib_path.parent().unwrap();
 
-    let mut cflags = OsString::from("-I");
-    cflags.push(builddir);
-    cflags.push(" ");
-    // We push the full path here to work around macos ld not supporting the -l:{filename} syntax
-    cflags.push(static_lib_path);
+    let mut cflags = OsString::new();
 
-    // We push the static_libs as CFLAGS as well to avoid mangling the options on msvc
-    cflags.push(" ");
-    cflags.push(static_libs);
+    for pkg in packages {
+        let static_lib_path = pkg.build_targets.static_lib.as_ref().unwrap();
+        let builddir = static_lib_path.parent().unwrap();
+
+        cflags.push("-I");
+        cflags.push(&builddir);
+        cflags.push(" ");
+
+        // We push the full path here to work around macos ld not supporting the -l:{filename} syntax
+        cflags.push(static_lib_path);
+
+        // We push the static_libs as CFLAGS as well to avoid mangling the options on msvc
+        cflags.push(" ");
+        cflags.push(&pkg.finger_print.static_libs);
+    }
 
     std::env::set_var("INLINE_C_RS_CFLAGS", cflags);
 
-    let err = ops::run_tests(&ws, &ops, &test_args)?;
+    let err = ops::run_tests(ws, &ops, &test_args)?;
     match err {
         None => Ok(()),
         Some(err) => {
-            let context = anyhow::format_err!("{}", err.hint(&ws, &ops.compile_opts));
+            let context = anyhow::format_err!("{}", err.hint(ws, &ops.compile_opts));
             let e = match err.code {
                 // Don't show "process didn't exit successfully" for simple errors.
                 Some(i) if is_simple_exit_code(i) => CliError::new(context, i),
@@ -1146,38 +1162,3 @@ pub fn ctest(
         }
     }
 }
-
-// Take the original cargo instance and save it as a separate env var if not already set.
-fn setup_env() {
-    if env::var("CARGO_C_CARGO").is_err() {
-        let cargo = env::var("CARGO").unwrap_or_else(|_| "cargo".to_owned());
-
-        std::env::set_var("CARGO_C_CARGO", cargo);
-    }
-}
-
-pub fn config_configure(config: &mut Config, args: &ArgMatches<'_>) -> CliResult {
-    let arg_target_dir = &args.value_of_path("target-dir", config);
-    let config_args: Vec<_> = args
-        .values_of("config")
-        .unwrap_or_default()
-        .map(String::from)
-        .collect();
-    config.configure(
-        args.occurrences_of("verbose") as u32,
-        args.is_present("quiet"),
-        args.value_of("color"),
-        args.is_present("frozen"),
-        args.is_present("locked"),
-        args.is_present("offline"),
-        arg_target_dir,
-        &args
-            .values_of_lossy("unstable-features")
-            .unwrap_or_default(),
-        &config_args,
-    )?;
-
-    // Make sure that the env-vars are correctly set at this point.
-    setup_env();
-    Ok(())
-}
