@@ -2,13 +2,12 @@ use std::path::{Component, Path, PathBuf};
 use structopt::clap::ArgMatches;
 
 use cargo::core::Workspace;
+use cargo_util::paths::{copy, create_dir_all};
 use semver::Version;
 
 use crate::build::*;
 use crate::build_targets::BuildTargets;
 
-use anyhow::Context;
-
 fn append_to_destdir(destdir: &Path, path: &Path) -> PathBuf {
     let mut joined = destdir.to_path_buf();
     for component in path.components() {
@@ -68,13 +67,6 @@ mod tests {
     }
 }
 
-fn copy<P: AsRef<Path>, Q: AsRef<Path>>(from: P, to: Q) -> anyhow::Result<u64> {
-    let from = from.as_ref();
-    let to = to.as_ref();
-    std::fs::copy(from, to)
-        .with_context(|| format!("Cannot copy {} to {}.", from.display(), to.display()))
-}
-
 pub(crate) enum LibType {
     So,
     Dylib,
@@ -167,76 +159,76 @@ impl UnixLibNames {
     }
 }
 
-pub fn cinstall(
-    ws: &Workspace,
-    capi_config: &CApiConfig,
-    build_targets: BuildTargets,
-    paths: InstallPaths,
-) -> anyhow::Result<()> {
-    use std::fs;
+pub fn cinstall(ws: &Workspace, packages: &[CPackage]) -> anyhow::Result<()> {
+    for pkg in packages {
+        let paths = &pkg.install_paths;
+        let capi_config = &pkg.capi_config;
+        let build_targets = &pkg.build_targets;
 
-    let destdir = &paths.destdir;
+        let destdir = &paths.destdir;
 
-    let mut install_path_lib = paths.libdir.clone();
-    if let Some(subdir) = &capi_config.library.install_subdir {
-        install_path_lib.push(subdir);
-    }
-
-    let install_path_lib = append_to_destdir(destdir, &install_path_lib);
-    let install_path_pc = append_to_destdir(destdir, &paths.pkgconfigdir);
-    let install_path_include = append_to_destdir(destdir, &paths.includedir);
-
-    fs::create_dir_all(&install_path_lib)?;
-    fs::create_dir_all(&install_path_pc)?;
-
-    ws.config()
-        .shell()
-        .status("Installing", "pkg-config file")?;
-    fs::copy(
-        &build_targets.pc,
-        install_path_pc.join(build_targets.pc.file_name().unwrap()),
-    )?;
-
-    if capi_config.header.enabled {
-        ws.config().shell().status("Installing", "header file")?;
-        for (from, to) in build_targets.extra.include.iter() {
-            let to = install_path_include.join(to);
-            fs::create_dir_all(to.parent().unwrap())?;
-            copy(from, to)?;
+        let mut install_path_lib = paths.libdir.clone();
+        if let Some(subdir) = &capi_config.library.install_subdir {
+            install_path_lib.push(subdir);
         }
-    }
 
-    if let Some(ref static_lib) = build_targets.static_lib {
-        ws.config().shell().status("Installing", "static library")?;
+        let install_path_lib = append_to_destdir(destdir, &install_path_lib);
+        let install_path_pc = append_to_destdir(destdir, &paths.pkgconfigdir);
+        let install_path_include = append_to_destdir(destdir, &paths.includedir);
+
+        create_dir_all(&install_path_lib)?;
+        create_dir_all(&install_path_pc)?;
+
+        ws.config()
+            .shell()
+            .status("Installing", "pkg-config file")?;
+
         copy(
-            static_lib,
-            install_path_lib.join(static_lib.file_name().unwrap()),
+            &build_targets.pc,
+            install_path_pc.join(build_targets.pc.file_name().unwrap()),
         )?;
-    }
 
-    if let Some(ref shared_lib) = build_targets.shared_lib {
-        ws.config().shell().status("Installing", "shared library")?;
-
-        let lib_name = &capi_config.library.name;
-        let lib_type = LibType::from_build_targets(&build_targets);
-        match lib_type {
-            LibType::So | LibType::Dylib => {
-                let lib =
-                    UnixLibNames::new(lib_type, lib_name, &capi_config.library.version).unwrap();
-                lib.install(&capi_config, &shared_lib, &install_path_lib)?;
+        if capi_config.header.enabled {
+            ws.config().shell().status("Installing", "header file")?;
+            for (from, to) in build_targets.extra.include.iter() {
+                let to = install_path_include.join(to);
+                create_dir_all(to.parent().unwrap())?;
+                copy(from, to)?;
             }
-            LibType::Windows => {
-                let install_path_bin = append_to_destdir(destdir, &paths.bindir);
-                fs::create_dir_all(&install_path_bin)?;
+        }
 
-                let lib_name = shared_lib.file_name().unwrap();
-                copy(shared_lib, install_path_bin.join(lib_name))?;
-                let impl_lib = build_targets.impl_lib.as_ref().unwrap();
-                let impl_lib_name = impl_lib.file_name().unwrap();
-                copy(impl_lib, install_path_lib.join(impl_lib_name))?;
-                let def = build_targets.def.as_ref().unwrap();
-                let def_name = def.file_name().unwrap();
-                copy(def, install_path_lib.join(def_name))?;
+        if let Some(ref static_lib) = build_targets.static_lib {
+            ws.config().shell().status("Installing", "static library")?;
+            copy(
+                static_lib,
+                install_path_lib.join(static_lib.file_name().unwrap()),
+            )?;
+        }
+
+        if let Some(ref shared_lib) = build_targets.shared_lib {
+            ws.config().shell().status("Installing", "shared library")?;
+
+            let lib_name = &capi_config.library.name;
+            let lib_type = LibType::from_build_targets(build_targets);
+            match lib_type {
+                LibType::So | LibType::Dylib => {
+                    let lib = UnixLibNames::new(lib_type, lib_name, &capi_config.library.version)
+                        .unwrap();
+                    lib.install(capi_config, shared_lib, &install_path_lib)?;
+                }
+                LibType::Windows => {
+                    let install_path_bin = append_to_destdir(destdir, &paths.bindir);
+                    create_dir_all(&install_path_bin)?;
+
+                    let lib_name = shared_lib.file_name().unwrap();
+                    copy(shared_lib, install_path_bin.join(lib_name))?;
+                    let impl_lib = build_targets.impl_lib.as_ref().unwrap();
+                    let impl_lib_name = impl_lib.file_name().unwrap();
+                    copy(impl_lib, install_path_lib.join(impl_lib_name))?;
+                    let def = build_targets.def.as_ref().unwrap();
+                    let def_name = def.file_name().unwrap();
+                    copy(def, install_path_lib.join(def_name))?;
+                }
             }
         }
     }
@@ -244,7 +236,7 @@ pub fn cinstall(
     Ok(())
 }
 
-#[derive(Debug, Hash)]
+#[derive(Debug, Hash, Clone)]
 pub struct InstallPaths {
     pub subdir_name: PathBuf,
     pub destdir: PathBuf,
